/*********************************************************************                                                                   **
**  Device:     A7130
**  File:		main.c
**  Author:		JPH
**  Target:		Winbond W77LE58
**  Tools:		ICE
**  Created:	2012-03-30
**  Description:
**  This file is a sample code for your reference.
**
**  Copyright (C) 2010 AMICCOM Electronics Corp.
**
**  Revision history:
**
*********************************************************************/
#include "define.h"
#include "A9108.h"
#include "a7130reg.h"
#include "Uti.h"

/*********************************************************************
**  I/O Declaration
*********************************************************************/
#define SCS       	P0_0		//spi SCS
#define SCK      	P0_1		//spi SCK
#define SDIO      	P0_2		//spi SDIO
#define CKO       	P0_3		//CKO
#define GIO1      	P0_6		//GPIO1
#define GIO2    	P0_7		//GPIO2
#define Button     	P1_7		//test Button

/*********************************************************************
**  Constant Declaration
*********************************************************************/
#define TIMEOUT     	50
#define t0hrel      	1000


/*********************************************************************
**  Global Variable Declaration
*********************************************************************/
Uint8 		data 	delayCnt;
Uint8		data 	timer;
Uint8		data 	TimeoutFlag;
Uint16		idata 	RxCnt;
Uint32		idata	Err_ByteCnt;
Uint32		idata	Err_BitCnt;
Uint16		idata	TimerCnt0;
Uint8      	data    *Uartptr;
Uint8		data	UartSendCnt;
Uint8		data	CmdBuf[12];
Uint8		idata	tmpbuf[80];
Uint8       idata   Err_Frame;
Uint8 		data	SubRxCnt;

const Uint8 code BitCount_Tab[16] = {0,1,1,2,1,2,2,3,1,2,2,3,2,3,3,4};
const Uint8 code ID_Tab[8]={0x34,0x75,0xC5,0x2A,0xC7,0x33,0x45,0xEA}; //ID code
const Uint8 code PageTab[8]={0x00,0x10,0x20,0x30,0x40,0x50,0x60,0x70};//page select
const Uint8 code PN9_Tab[]=
{   0xFF,0x83,0xDF,0x17,0x32,0x09,0x4E,0xD1,
    0xE7,0xCD,0x8A,0x91,0xC6,0xD5,0xC4,0xC4,
    0x40,0x21,0x18,0x4E,0x55,0x86,0xF4,0xDC,
    0x8A,0x15,0xA7,0xEC,0x92,0xDF,0x93,0x53,
    0x30,0x18,0xCA,0x34,0xBF,0xA2,0xC7,0x59,
    0x67,0x8F,0xBA,0x0D,0x6D,0xD8,0x2D,0x7D,
    0x54,0x0A,0x57,0x97,0x70,0x39,0xD2,0x7A,
    0xEA,0x24,0x33,0x85,0xED,0x9A,0x1D,0xE0
};	// This table are 64bytes PN9 pseudo random code.

#define A7130_DR4M
//#define A7130_DR3M
//#define A7130_DR2M
//#define A7130_DR1M

#ifdef A7130_DR4M
const Uint8 code A7130Config[]=
{
	0x00, //RESET register,			only reset, not use on config
	0x62, //MODE register,
	0x00, //CALIBRATION register,
	0x3F, //FIFO1 register,
	0x00, //FIFO2 register,
	0x00, //FIFO register,			for fifo read/write
	0x00, //IDDATA register,		for idcode
	0x00, //RCOSC1 register,
	0x00, //RCOSC2 register,
	0x00, //RCOSC3 register,
	0x00, //CKO register,
	0x01, //GPIO1 register
	0x1D, //GPIO2 register,
	0x9F, //DATARATECLOCK register,
	0x00, //PLL1 register,
	0x0E, //PLL2 register, 			RFbase 2400.001MHz
	0x96, //PLL3 register,
	0x00, //PLL4 register,
	0x04, //PLL5 register,
	0x3C, //ChannelGroup1 register,
	0x78, //ChannelGroup1 register,
	0xAF, //TX1 register,
	0x40, //TX2 register,
	0x10, //DELAY1 register,
	0x60, //DELAY2 register,
	0x72, //RX register,
	0x6F, //RXGAIN1 register,
	0xC2, //RXGAIN2 register,
	0x3D, //RXGAIN3 register,
	0xE7, //RXGAIN4 register,
	0x00, //RSSI register,
	0xF1, //ADC register,
	0x07, //CODE1 register,
	0x06, //CODE2 register,
	0x2A, //CODE3 register,
	0x60, //IFCAL1 register,
	0xFF, //IFCAL2 register,		**0x7F setting for 17dBm TX power module**
	0x80, //VCOCCAL register,
	0xD0, //VCOCAL1 register,
	0x00, //VCOCAL2 register,
	0x70, //VCO deviation 1 register,
	0x00, //VCO deviation 2 register,
	0x00, //DSA register,
	0xDC, //VCO Modulation delay register,
	0xF0, //BATTERY register,
	0x37, //TXTEST register,				
	0x47, //RXDEM1 register,
	0xF7, //RXDEM2 register,
	0xF0, //CPC1 register,
	0x37, //CPC2 register,
	0x55, //CRYSTAL register,
	0x15, //PLLTEST register,
	0x15, //VCOTEST register,
	0x00, //RF Analog register,
	0x00, //Key data register,
	0x77, //Channel select register,
	0x00, //ROM register,
	0x00, //DataRate register,
	0x00, //FCR register,
	0x00, //ARD register,
	0x00, //AFEP register,
	0x00, //FCB register,
	0x00, //KEYC register,
	0x00  //USID register,
};

const Uint8 code A7130_Addr2A_Config[]=
{
	0x34, //page0,					**0x74 setting for 17dBm TX power module**
	0x49, //page1,
	0x00, //Page2,
	0x80, //page3,
	0x80, //page4,
	0x00, //page5,
	0x0A, //page6,
	0x00, //page7,
};

const Uint8 code A7130_Addr38_Config[]=
{
	0x00, //page0,
	0x10, //page1,
	0x20, //page2,
	0x24, //page3,
	0x20, //page4,
};
#endif
#ifdef A7130_DR3M
const Uint8 code A7130Config[]=
{
	0x00, //RESET register,			only reset, not use on config
	0x62, //MODE register,
	0x00, //CALIBRATION register,
	0x3F, //FIFO1 register,
	0x00, //FIFO2 register,
	0x00, //FIFO register,			for fifo read/write
	0x00, //IDDATA register,		for idcode
	0x00, //RCOSC1 register,
	0x00, //RCOSC2 register,
	0x00, //RCOSC3 register,
	0x00, //CKO register,
	0x01, //GPIO1 register
	0x1D, //GPIO2 register,
	0x5F, //DATARATECLOCK register,
	0x00, //PLL1 register,
	0x0E, //PLL2 register, 			RFbase 2400.001MHz
	0x96, //PLL3 register,
	0x00, //PLL4 register,
	0x04, //PLL5 register,
	0x3C, //ChannelGroup1 register,
	0x78, //ChannelGroup1 register,
	0xAE, //TX1 register,
	0x60, //TX2 register,
	0x12, //DELAY1 register,
	0x40, //DELAY2 register,
	0x70, //RX register,
	0x6F, //RXGAIN1 register,
	0xC2, //RXGAIN2 register,
	0x2D, //RXGAIN3 register,
	0xC7, //RXGAIN4 register,
	0x00, //RSSI register,
	0xF1, //ADC register,
	0x07, //CODE1 register,
	0x05, //CODE2 register,
	0x2A, //CODE3 register,
	0x20, //IFCAL1 register,
	0xFF, //IFCAL2 register,		**0x7F setting for 17dBm TX power module**
	0x80, //VCOCCAL register,
	0xD0, //VCOCAL1 register,
	0x00, //VCOCAL2 register,
	0xB0, //VCO deviation 1 register,
	0x00, //VCO deviation 2 register,
	0x00, //DSA register,
	0xAF, //VCO Modulation delay register,
	0xF0, //BATTERY register,
	0x37, //TXTEST register,				
	0x47, //RXDEM1 register,
	0xF7, //RXDEM2 register,
	0xF0, //CPC1 register,
	0x37, //CPC2 register,
	0x55, //CRYSTAL register,
	0x15, //PLLTEST register,
	0x15, //VCOTEST register,
	0x00, //RF Analog register,
	0x00, //Key data register,
	0x57, //Channel select register,
	0x00, //ROM register,
	0x00, //DataRate register,
	0x00, //FCR register,
	0x00, //ARD register,
	0x00, //AFEP register,
	0x00, //FCB register,
	0x00, //KEYC register,
	0x00  //USID register,
};

const Uint8 code A7130_Addr2A_Config[]=
{
	0x34, //page0,					**0x74 setting for 17dBm TX power module**
	0x49, //page1,
	0x00, //Page2,
	0x80, //page3,
	0x80, //page4,
	0x00, //page5,
	0x0A, //page6,
	0xC0, //page7,
};

const Uint8 code A7130_Addr38_Config[]=
{
	0x00, //page0,
	0x10, //page1,
	0x20, //page2,
	0x24, //page3,
	0x20, //page4,
};
#endif

#ifdef A7130_DR2M
const Uint8 code A7130Config[]=
{
	0x00, //RESET register,			only reset, not use on config
	0x62, //MODE register,
	0x00, //CALIBRATION register,
	0x3F, //FIFO1 register,
	0x00, //FIFO2 register,
	0x00, //FIFO register,			for fifo read/write
	0x00, //IDDATA register,		for idcode
	0x00, //RCOSC1 register,
	0x00, //RCOSC2 register,
	0x00, //RCOSC3 register,
	0x00, //CKO register,
	0x01, //GPIO1 register
	0x1D, //GPIO2 register,
	0x5F, //DATARATECLOCK register,
	0x00, //PLL1 register,
	0x0E, //PLL2 register, 			RFbase 2400.001MHz
	0x96, //PLL3 register,
	0x00, //PLL4 register,
	0x04, //PLL5 register,
	0x3C, //ChannelGroup1 register,
	0x78, //ChannelGroup1 register,
	0xAE, //TX1 register,
	0x40, //TX2 register,
	0x12, //DELAY1 register,
	0x40, //DELAY2 register,
	0x70, //RX register,
	0x6F, //RXGAIN1 register,
	0xC2, //RXGAIN2 register,
	0x1D, //RXGAIN3 register,
	0xC7, //RXGAIN4 register,
	0x00, //RSSI register,
	0xF1, //ADC register,
	0x07, //CODE1 register,
	0x05, //CODE2 register,
	0x2A, //CODE3 register,
	0x20, //IFCAL1 register,
	0xFF, //IFCAL2 register,		**0x7F setting for 17dBm TX power module**
	0x80, //VCOCCAL register,
	0xD0, //VCOCAL1 register,
	0x00, //VCOCAL2 register,
	0x70, //VCO deviation 1 register,
	0x00, //VCO deviation 2 register,
	0x00, //DSA register,
	0x9C, //VCO Modulation delay register,
	0xF0, //BATTERY register,
	0x37, //TXTEST register,				
	0x47, //RXDEM1 register,
	0xF7, //RXDEM2 register,
	0xF0, //CPC1 register,
	0x37, //CPC2 register,
	0x55, //CRYSTAL register,
	0x15, //PLLTEST register,
	0x15, //VCOTEST register,
	0x00, //RF Analog register,
	0x00, //Key data register,
	0x37, //Channel select register,
	0x00, //ROM register,
	0x00, //DataRate register,
	0x00, //FCR register,
	0x00, //ARD register,
	0x00, //AFEP register,
	0x00, //FCB register,
	0x00, //KEYC register,
	0x00  //USID register,
};

const Uint8 code A7130_Addr2A_Config[]=
{
	0x34, //page0,					**0x74 setting for 17dBm TX power module**
	0x49, //page1,
	0x00, //Page2,
	0x80, //page3,
	0x80, //page4,
	0x00, //page5,
	0x0A, //page6,
	0x40, //page7,
};

const Uint8 code A7130_Addr38_Config[]=
{
	0x00, //page0,
	0x10, //page1,
	0x20, //page2,
	0x24, //page3,
	0x20, //page4,
};
#endif

#ifdef A7130_DR1M
const Uint8 code A7130Config[]=
{
	0x00, //RESET register,			only reset, not use on config
	0x62, //MODE register,
	0x00, //CALIBRATION register,
	0x3F, //FIFO1 register,
	0x00, //FIFO2 register,
	0x00, //FIFO register,			for fifo read/write
	0x00, //IDDATA register,		for idcode
	0x00, //RCOSC1 register,
	0x00, //RCOSC2 register,
	0x00, //RCOSC3 register,
	0x00, //CKO register,
	0x01, //GPIO1 register
	0x1D, //GPIO2 register,
	0x5F, //DATARATECLOCK register,
	0x00, //PLL1 register,
	0x0E, //PLL2 register, 			RFbase 2400.001MHz
	0x96, //PLL3 register,
	0x00, //PLL4 register,
	0x04, //PLL5 register,
	0x3C, //ChannelGroup1 register,
	0x78, //ChannelGroup1 register,
	0xAD, //TX1 register,
	0x40, //TX2 register,
	0x12, //DELAY1 register,
	0x40, //DELAY2 register,
	0x70, //RX register,
	0x6F, //RXGAIN1 register,
	0xC2, //RXGAIN2 register,
	0x0D, //RXGAIN3 register,
	0xC7, //RXGAIN4 register,
	0x00, //RSSI register,
	0xF1, //ADC register,
	0x07, //CODE1 register,
	0x06, //CODE2 register,
	0x2A, //CODE3 register,
	0x20, //IFCAL1 register,
	0xFF, //IFCAL2 register,		**0x7F setting for 17dBm TX power module**
	0x80, //VCOCCAL register,
	0xD0, //VCOCAL1 register,
	0x00, //VCOCAL2 register,
	0x30, //VCO deviation 1 register,
	0x00, //VCO deviation 2 register,
	0x00, //DSA register,
	0x89, //VCO Modulation delay register,
	0xF0, //BATTERY register,
	0x37, //TXTEST register,				
	0x47, //RXDEM1 register,
	0xF7, //RXDEM2 register,
	0xF0, //CPC1 register,
	0x37, //CPC2 register,
	0x55, //CRYSTAL register,
	0x15, //PLLTEST register,
	0x15, //VCOTEST register,
	0x00, //RF Analog register,
	0x00, //Key data register,
	0x17, //Channel select register,
	0x00, //ROM register,
	0x00, //DataRate register,
	0x00, //FCR register,
	0x00, //ARD register,
	0x00, //AFEP register,
	0x00, //FCB register,
	0x00, //KEYC register,
	0x00  //USID register,
};

const Uint8 code A7130_Addr2A_Config[]=
{
	0x34, //page0,					**0x74 setting for 17dBm TX power module**
	0x49, //page1,
	0x00, //Page2,
	0x80, //page3,
	0x80, //page4,
	0x00, //page5,
	0x0A, //page6,
	0x40, //page7,
};

const Uint8 code A7130_Addr38_Config[]=
{
	0x00, //page0,
	0x10, //page1,
	0x20, //page2,
	0x24, //page3,
	0x20, //page4,
};
#endif
/*********************************************************************
**  function Declaration
*********************************************************************/
void InitTimer0(void);
void initUart0(void);
void Timer0ISR (void);
void Uart0Isr(void);
void A7130_Reset(void);
void A7130_WriteReg_Page(Uint8 addr, Uint8 wbyte, Uint8 page);
Uint8 A7130_ReadReg_Page(Uint8 addr, Uint8 page);
void A7130_WriteReg(Uint8, Uint8);
Uint8 A7130_ReadReg(Uint8);
void ByteSend(Uint8 src);
Uint8 ByteRead(void);
void A7130_WriteID(void);
void A7130_WriteFIFO(void);
void initRF(void);
void A7130_Config(void);
void A7130_Cal(void);
void RxPacket(void);
void StrobeCmd(Uint8);
void SetCH(Uint8);
void Send4Bit(Uint8 src);
void A7130_Autoresend(void);
void A7130_Autoack(void);
void WOR_enable(void);

/*********************************************************************
* main loop
*********************************************************************/
void main(void)
{
    //initHW
    P0 = 0xFF;
    P1 = 0xFF;
    P3 = 0xFF;

	P0OE =0x3F;
	P1OE =0xFF;
	P3OE =0xFF;
	
	P0PUN   = 0xFF;
	P1PUN   = 0xFF;
	P3PUN   = 0xFF;
	
    InitTimer0();
    initUart0();
    TR0=1; //timer0 on
    EA=1; //enable interrupt

	if ((P3 & 0x04)==0x04) //if P4.2=1, master
	{
		initRF(); //init RF
    StrobeCmd(CMD_STBY);

		SetCH(100); 
		 /* 
		     How to set RF channel
             RF base Freq : 2400.001MHz
             Channel step : 0.5MHz
             SetCH        : 100
             RF Frequency = RF base Freq + (Channel step * SetCH) = 2400.001MHz  + (   0.5MHz    *  100 ) = 2450.001MHz 
            */
		SCS=0;
		ByteSend(FIFO1_REG);//fifo length upto 4K bytes
		ByteSend(0x3F);	//low byte
		ByteSend(0x00);	//high byte
		SCS=1;

		
		P1_1=0;
		while(1)
		{
			A7130_WriteFIFO(); //write data to tx fifo
			StrobeCmd(CMD_TX); //entry tx & transmit
			while(GIO1); //wait transmit completed
			P1_3=0;
			Delay10ms(1);
			
//			StrobeCmd(CMD_RX); //entry rx

//			timer=0;
//			TimeoutFlag=0;
//			while(GIO1==1 && TimeoutFlag==0); //wait receive completed
//			if (TimeoutFlag)
//			{
//			    StrobeCmd(CMD_STBY); //exit rx mode
//					continue;
//			}

//			RxPacket();
//			Delay10ms(1);
    }
	}
	else //if P4.2=0, slave
	{
		initRF(); //init RF
		StrobeCmd(CMD_STBY);

		RxCnt = 0;
		Err_ByteCnt = 0;
		Err_BitCnt = 0;

		SetCH(100); //freq 2450.001MHz
    SCS=0;
		ByteSend(FIFO1_REG);//fifo length upto 4K bytes
		ByteSend(0x3F);	//low byte
		ByteSend(0x00);	//high byte
		SCS=1;
		
		while(1)
		{
			P1=0;	
			WOR_enable();
			P1=0xFF;
			while(1);

//			StrobeCmd(CMD_RX);
//			while(GIO1); //wait receive completed
//			RxPacket();

//			A7130_WriteFIFO(); //write data to tx fifo
//			StrobeCmd(CMD_TX); //entry tx
//			while(GIO1); //wait transmit completed
		}
	}
}

/************************************************************************
**  init Timer0
************************************************************************/
void InitTimer0(void)
{
    TR0 = 0;
    TMOD =(TMOD & 0xF0)|0x01; //timer0 mode=1
    TH0 = (65536-t0hrel)>>8; // Reload Timer0 high byte,low byte
    TL0 = 65536-t0hrel;
    TF0 = 0; // Clear any pending Timer0 interrupts
    ET0 = 1; // Enable Timer0 interrupt
}

/************************************************************************
**  Timer0ISR
************************************************************************/
void Timer0ISR (void) interrupt 1
{
    TF0 = 0; // Clear Timer0 interrupt
    TH0 = (65536-t0hrel)>>8; // Reload Timer0 high byte,low byte
    TL0 = 65536-t0hrel;

    timer++;
    if (timer == TIMEOUT)
    {
        TimeoutFlag=1;
    }

    TimerCnt0++;
    if (TimerCnt0 == 500)
    {
        TimerCnt0=0;
        CmdBuf[0]=0xF1;

        memcpy(&CmdBuf[1], &RxCnt, 2);
        memcpy(&CmdBuf[3], &SubRxCnt, 4);
        memcpy(&CmdBuf[7], &Err_BitCnt, 4);
        memcpy(&CmdBuf[11], &Err_Frame, 1);

        UartSendCnt=12;
		Uartptr=&CmdBuf[0];
        SBUF=CmdBuf[0];
    }
}

/************************************************************************
**  Init Uart0
************************************************************************/
void initUart0(void)
{
    TH1 = 0xFD; //BaudRate 9600;
    TL1 = 0xFD;
    SCON = 0x40;
    TMOD = (TMOD & 0x0F) | 0x20;
    REN = 1;
    TR1 = 1;
    ES = 1;
}

/************************************************************************
**  Uart0 ISR
************************************************************************/
void Uart0Isr(void) interrupt 4 using 3
{
    if (TI==1)
    {
        TI=0;
        UartSendCnt--;
        if(UartSendCnt !=0)
        {
            Uartptr++;
            SBUF = *Uartptr;
        }
    }
}

/************************************************************************
**  Reset_RF
************************************************************************/
void A7130_Reset(void)
{
	A7130_WriteReg(MODE_REG, 0x00); //reset RF chip
}

/************************************************************************
**  WriteID
************************************************************************/
void A7130_WriteID(void)
{
	Uint8 i;
	Uint8 d1,d2,d3,d4,d5,d6,d7,d8;
	Uint8 addr;

	addr = IDCODE_REG; //send address 0x06, bit cmd=0, r/w=0
	SCS = 0;
	ByteSend(addr);
	for (i=0; i < 8; i++)
		ByteSend(ID_Tab[i]);
	SCS = 1;

	addr = IDCODE_REG | 0x40; //send address 0x06, bit cmd=0, r/w=1
	SCS=0;
	ByteSend(addr);
	d1=ByteRead();
	d2=ByteRead();
	d3=ByteRead();
	d4=ByteRead();
	d5=ByteRead();
	d6=ByteRead();
	d7=ByteRead();
	d8=ByteRead();
	SCS=1;
}

/*********************************************************************
** A7130_ReadReg_Page
*********************************************************************/
Uint8 A7130_ReadReg_Page(Uint8 addr, Uint8 page)
{
	Uint8 tmp;

	A7130_WriteReg(RFANALOG_REG, A7130Config[RFANALOG_REG] | PageTab[page]);//page select
	tmp = A7130_ReadReg(addr);
	return tmp;
}

/*********************************************************************
** A7130_WriteReg_Page
*********************************************************************/
void A7130_WriteReg_Page(Uint8 addr, Uint8 wbyte, Uint8 page)
{
	A7130_WriteReg(RFANALOG_REG, A7130Config[RFANALOG_REG] | PageTab[page]);//page select
	A7130_WriteReg(addr, wbyte);
}

/************************************************************************
**  A7130_WriteReg
************************************************************************/
void A7130_WriteReg(Uint8 addr, Uint8 dataByte)
{
    Uint8 i;

	SCS = 0;

    addr |= 0x00; //bit cmd=0,r/w=0
    for(i = 0; i < 8; i++)
    {
        if(addr & 0x80)
            SDIO = 1;
        else
            SDIO = 0;

        SCK = 1;
        _nop_();
        SCK = 0;
        addr = addr << 1;
    }
    _nop_();

    //send data byte
    for(i = 0; i < 8; i++)
    {
        if(dataByte & 0x80)
            SDIO = 1;
        else
            SDIO = 0;

        SCK = 1;
        _nop_();
        SCK = 0;
        dataByte = dataByte<< 1;
    }

    SCS = 1;
}

/************************************************************************
**  A7130_ReadReg
************************************************************************/
Uint8 A7130_ReadReg(Uint8 addr)
{
    Uint8 i;
    Uint8 tmp;

	SCS = 0;
    addr |= 0x40; //bit cmd=0,r/w=1
    for(i = 0; i < 8; i++)
    {
        if(addr & 0x80)
            SDIO = 1;
        else
            SDIO = 0;

		_nop_();
        SCK = 1;
        _nop_();
        SCK = 0;

        addr = addr << 1;
    }

    _nop_();
    SDIO = 1;
	P0OE&=0xFB;
    //read data
    for(i = 0; i < 8; i++)
    {
        if(SDIO)
            tmp = (tmp << 1) | 0x01;
        else
            tmp = tmp << 1;

        SCK = 1;
        _nop_();
        SCK = 0;
    }
	P0OE|=0x04;		
    SCS = 1;
    return tmp;
}

/************************************************************************
**  ByteSend
************************************************************************/
void ByteSend(Uint8 src)
{
    Uint8 i;

    for(i = 0; i < 8; i++)
    {
        if(src & 0x80)
            SDIO = 1;
        else
            SDIO = 0;

		_nop_();
        SCK = 1;
        _nop_();
        SCK = 0;
        src = src << 1;
    }
}

/************************************************************************
**  ByteRead
************************************************************************/
Uint8 ByteRead(void)
{
	Uint8 i,tmp;

	P0OE&=0xFB;		
    SDIO = 1; //sdio pull high
    for(i = 0; i < 8; i++)
    {
        if(SDIO)
            tmp = (tmp << 1) | 0x01;
        else
            tmp = tmp << 1;

        SCK = 1;
        _nop_();
        SCK = 0;
    }
	P0OE|=0x04;		
    return tmp;
}

/************************************************************************
**  Send4Bit
************************************************************************/
void Send4Bit(Uint8 src)
{
    Uint8 i;

    for(i = 0; i < 4; i++)
    {
        if(src & 0x80)
            SDIO = 1;
        else
            SDIO = 0;

		_nop_();
        SCK = 1;
        _nop_();
        SCK = 0;
        src = src << 1;
    }
}

/*********************************************************************
** SetCH
*********************************************************************/
void SetCH(Uint8 ch)
{
	A7130_WriteReg(PLL1_REG, ch); //RF freq = RFbase + (CH_Step * ch)
}

/*********************************************************************
** initRF
*********************************************************************/
void initRF(void)
{
    //init io pin
    SCS = 1;
    SCK = 0;
    SDIO = 1;
    CKO = 1;
    GIO1 = 1;
    GIO2 = 1;

    A7130_Reset(); //reset A7105 RF chip
	A7130_WriteID(); //write ID code
	A7130_Config(); //config A7105 chip
	A7130_Cal(); //calibration IF,VCO,VCOC
}

/*********************************************************************
** A7130_WriteFIFO
*********************************************************************/
void A7130_WriteFIFO(void)
{
	Uint8 i;
	Uint8 cmd;

	cmd = FIFO_REG; //send address 0x05, bit cmd=0, r/w=0
    SCS=0;
	ByteSend(cmd);
	for(i=0; i <64; i++)
		ByteSend(PN9_Tab[i]);
	SCS=1;
}

/*********************************************************************
** Strobe Command
*********************************************************************/
void StrobeCmd(Uint8 cmd)
{
	SCS = 0;
	Send4Bit(cmd);
	SCS = 1;
}

/*********************************************************************
** RxPacket
*********************************************************************/
void RxPacket(void)
{
    Uint8 i;
    Uint8 recv;
    Uint8 tmp;
    Uint8 cmd;

	RxCnt++;
	cmd = FIFO_REG | 0x40; //address 0x05, bit cmd=0, r/w=1

    SCS=0;
	ByteSend(cmd);
	for(i=0; i <64; i++)
	{
		recv = ByteRead();
		tmpbuf[i]=recv;
		if((recv ^ PN9_Tab[i])!=0)
        {
            tmp = recv ^ PN9_Tab[i];
            Err_BitCnt += (BitCount_Tab[tmp>>4] + BitCount_Tab[tmp & 0x0F]);
        }
    }
	SCS=1;
}

/*********************************************************************
** Err_State
*********************************************************************/
void Err_State(void)
{
    //ERR display
    //Error Proc...
    //...
    while(1);
}

/*********************************************************************
** CHGroupCal
*********************************************************************/
void CHGroupCal(Uint8 ch)
{
    Uint8 tmp;
    Uint8 vb,vbcf;
    Uint8 deva,adev;

	A7130_WriteReg(PLL1_REG, ch);

	A7130_WriteReg(CALIBRATION_REG, 0x0C);
	do
	{
		tmp = A7130_ReadReg(CALIBRATION_REG);
		tmp &= 0x0C;
	}
	while (tmp);

    //for check
    tmp = A7130_ReadReg(VCOCAL1_REG);
	vb = tmp & 0x07;
	vbcf = (tmp >>3) & 0x01;

	tmp = A7130_ReadReg(VCODEVCAL1_REG);
	deva = tmp;
	
	tmp = A7130_ReadReg(VCODEVCAL2_REG);
	adev = tmp;
	if (adev<41 || adev >68)
		Err_State();//error

	if (vbcf)
	   Err_State();//error
}

/*********************************************************************
** calibration
*********************************************************************/
void A7130_Cal(void)
{
	Uint8 tmp, loopCnt;
	Uint8 fb, fbcf, fcd, vcb, vccf, rhc, rlc;

    StrobeCmd(CMD_PLL); //calibration @PLL state

	//calibration IF procedure
	loopCnt = 0;
	//A7130_WriteReg(RXGAIN3_REG, 0x1D);
	while(1)
	{
		A7130_WriteReg(CALIBRATION_REG, 0x02);
		do
		{
			tmp = A7130_ReadReg(CALIBRATION_REG);
			tmp &= 0x02;
		}
		while(tmp);

		tmp = A7130_ReadReg(IFCAL1_REG);
		fb = tmp & 0x0F;
		if (fb>3 && fb<11)
			break;

		loopCnt++;
		if (loopCnt>=10)
			break;
	}
	//A7130_WriteReg(RXGAIN3_REG, 0x3D);

	if (loopCnt >=10)
		Err_State();

	//calibration RSSI, VCC procedure
	A7130_WriteReg(CALIBRATION_REG, 0x11);
	do
	{
		tmp = A7130_ReadReg(CALIBRATION_REG);
		tmp &= 0x11;
	}
	while(tmp);
	
	//calibration VBC,VDC procedure
	CHGroupCal(30); //calibrate channel group Bank I
	CHGroupCal(90); //calibrate channel group Bank II
	CHGroupCal(150); //calibrate channel group Bank III
	StrobeCmd(CMD_STBY); //return to STBY state

	//for check
	tmp = A7130_ReadReg(IFCAL1_REG);
	fb = tmp & 0x0F;
	fbcf = (tmp >>4) & 0x01;

	tmp = A7130_ReadReg(IFCAL2_REG);
	fcd = tmp & 0x1F;

	tmp = A7130_ReadReg(VCOCCAL_REG) & 0x1F;
	vcb = tmp & 0x0F;
	vccf = (tmp >>4) & 0x01;

	rhc = A7130_ReadReg(RXGAIN2_REG);
	rlc = A7130_ReadReg(RXGAIN3_REG);

	if (fbcf || vccf)
	   Err_State();
}

/*********************************************************************
** A7130_Config
*********************************************************************/
void A7130_Config(void)
{
	Uint8 i;

    //0x00 mode register, for reset
    //0x05 fifo data register
    //0x06 id code register
    //0x3F USID register, read only
    //0x36 key data, 16 bytes
    //0x3D FCB register,4 bytes

	A7130_WriteReg(0x1C, A7130Config[0x1C]);
	A7130_WriteReg(0x0D, A7130Config[0x0D]);
	
    for (i=0x01; i<=0x04; i++)
	    A7130_WriteReg(i, A7130Config[i]);

	for (i=0x07; i<=0x29; i++)
		A7130_WriteReg(i, A7130Config[i]);

	for (i=0; i<=7; i++)//0x2A DAS
		A7130_WriteReg_Page(0x2A, A7130_Addr2A_Config[i], i);

	for (i=0x2B; i<=0x35; i++)
		A7130_WriteReg(i, A7130Config[i]);

	A7130_WriteReg(0x37, A7130Config[0x37]);

	for (i=0; i<=4; i++)//0x38 ROM
		A7130_WriteReg_Page(0x38, A7130_Addr38_Config[i], i);

	for (i=0x39; i<=0x3C; i++)
		A7130_WriteReg(i, A7130Config[i]);

	A7130_WriteReg(0x3E, A7130Config[0x3E]);
}
/*********************************************************************
** A7130_Auto Resend
*********************************************************************/
void A7130_Autoresend(void)
{
	StrobeCmd(CMD_STBY);
	
	A7130_WriteReg(0x0B, 0x39);//GIO1=VPOAK, 
	A7130_WriteReg(0x0C, 0x01);//GIO2=WTR
	A7130_WriteReg(0x0A, 0x02);//CKO=DCK
	
	A7130_WriteReg(0x3A, 0x3D); //enable Auto Resend, by event, fixed, ARC=15
	A7130_WriteReg(0x3B, 0x1E);//ARD=30
	//ACK format = Preamble + ID = 8bytes
    //so set ARD=30, RX time=30*250us=7.75ms
	    while(1)
    {
        A7130_WriteFIFO();
        StrobeCmd(CMD_TX);      //entry TX mode
        Delay10us(1);
        while(GIO2);            //wait transmit completed

        //check VPOAK
        if(GIO1)
        {
            //ACK ok
        }
        else
        {
            //NO ACK
        }

        StrobeCmd(CMD_STBY);    //clear VPOAK(GIO1 signal)

        Delay10ms(1);
    }
	
}
/*********************************************************************
** A7130_Auto Ack
*********************************************************************/
void A7130_Autoack(void)
{
	StrobeCmd(CMD_STBY);
    
	A7130_WriteReg(0x0B, 0x05);//GIO1=FSYNC, 
	A7130_WriteReg(0x0C, 0x01);//GIO2=WTR
	A7130_WriteReg(0x0A, 0x0A);//CKO=RCK

    A7130_WriteReg(0x3A, 0x02);   //enable Auto ACK

    while(1)
    {
        StrobeCmd(CMD_RX);      //entry RX mode
        Delay10us(1);           
        while(GIO2);            //wait receive completed

        RxPacket();
    }
}

//-----------------------------------------------------------------------------
// Function     : RCOSC_Calibration(void)
//
// Description  : Starup and calibration RC-OSC
//
// Parament     : None
//
// Return       : 0 = PASS
//              : others = FAIL
//-----------------------------------------------------------------------------
Uint8 RCOSC_Calibration(void)
{
    Uint8 tmp, rcoc, rcot, retry;

    for(rcot = 0; rcot < 4; rcot++)
    {
        for(retry = 0; retry < 3; retry ++)
        {
            A7130_WriteReg(RCOSC3_REG, 0x00);                   // RC-OSC disable
            A7130_WriteReg(RCOSC3_REG, (0x04 | (rcot << 4)));   // RCOSC_E=1, CALW=0. RC-OSC enable (startup).
            rcoc = A7130_ReadReg(RCOSC1_REG);                   // read RCOC before calibration
            A7130_WriteReg(RCOSC3_REG, (0x0C | (rcot << 4)));   // RCOSC_E=1, CALW=1. RC-OSC calibration begin.

            do
            {
                tmp = A7130_ReadReg(RCOSC3_REG) & 0x08;
            } while(tmp);                                       // wait RC-OSC calibration finish.
            
            tmp = A7130_ReadReg(RCOSC1_REG);
            if(tmp != rcoc)                                     // check RCOC.
            {
                rcoc = tmp;
                if(rcoc >= 20)
                {
                    return 0;
                }
            }
        }
    }

    return 0xFF;
}



/*********************************************************************
** WOR_enable 
*********************************************************************/
void WOR_enable(void)
{
	A7130_WriteReg(GPIO1_REG, 0x01);	//GIO1 as WTR
	A7130_WriteReg(GPIO2_REG, 0x11);	//GIO1 as Wake up signal
	
	A7130_WriteReg(CODE1_REG,A7130Config[CODE1_REG]&0xEF); //Disable CRC
	
	//Real WOR Active Period = (WOR_AC[5:0]+1) x 244us ? 600us(X'TAL and Regulator settling time)

	//Note : Be aware that X?tal settling time requirement includes initial tolerance, 
	//       temperature drift, aging and crystal loading.
	
	//Real WOR Sleep Period = (WOR_SL[9:0]+1) x 7.8ms
	
	A7130_WriteReg(RCOSC1_REG, 0x0A);	//setup WOR Sleep time and RX time
	A7130_WriteReg(RCOSC2_REG, 0x14);
	StrobeCmd(CMD_STBY);				//entry standby mode

	//WOR Calibration
    RCOSC_Calibration();
	
	A7130_WriteReg(MODECTRL_REG, A7130Config[MODECTRL_REG] | 0x08);	//WORE=1 to enable WOR function
	while(GIO2==0);		//Stay in WOR mode until receiving ID code(sync ok)

	A7130_WriteReg(MODECTRL_REG, A7130Config[MODECTRL_REG]);	//WORE=0 to disable WOR function
	RxPacket();
}